import camelCase from "lodash/camelCase";
import kebabCase from "lodash/kebabCase";
import { inject, computed, useAttrs, toRef } from "vue";
import omit from "lodash/omit";
import { RiverConfig } from "../types";
import { normalizeClassAsArray } from "../utils/box";
import defaultConfig from "../defaultConfig";

export function useBoxProps(defaultProps: Record<string, any> = {}) {
  const attrs = useAttrs();
  const config = inject<RiverConfig>("$riverBoxConfig", defaultConfig);

  return computed(() => {
    let attributes = { ...defaultProps, ...attrs };
    let classes = normalizeClassAsArray(attributes.class);

    // Flatten attributes into a class list compatible with vue's
    // v-bind:class binding
    for (let attr in attributes) {
      if (
        config.boxProps.list.indexOf(camelCase(attr)) !== -1 ||
        config.boxProps.list.indexOf(kebabCase(attr)) !== -1
      ) {
        classes = [
          ...classes,
          ...config.boxProps.parser(attr, attributes[attr]),
        ];
      }
    }

    // Remove polymorphic tag prop from attributes as these will be forwarded
    // directly to the resulting html tag
    delete attributes.as;
    delete attributes.is;

    return {
      as: attrs.as ?? attrs.is ?? "div",
      attrs: {
        ...omit(attributes, config.boxProps.list),
        class: classes.length ? classes : null,
      },
    };
  });
}
